@@ -98,7 +98,7 @@ sub Read
 					{	@v= map $sub->($_,$id,$key,$field), @v;
 						next unless @v;
 					}
-					if ($join)	{ push @$value, @v; }
+					if ($join)	{ push @$value, grep defined, @v; }
 					else		{ $value= $v[0]; last; }
 				}
 				next unless defined $value;
@@ -985,10 +985,12 @@ use Gtk2;
 use base 'Gtk2::SpinButton';
 
 sub new
-{	my ($class,$field,$IDs,$max,$digits,$specialmode) = @_;
-	$max||=10000000;
-	$digits||=0;
-	my $adj=Gtk2::Adjustment->new(0,0,$max,1,10,0);
+{	my ($class,$field,$IDs,%opt) = @_;	#possible options in %opt : signed digits min max mode
+	my $mode=$opt{mode}||'';
+	my $max= $opt{max} || 10000000;
+	my $min= $opt{min} || ($opt{signed} ? -$max : 0);
+	my $digits= $opt{digits} || 0;
+	my $adj=Gtk2::Adjustment->new(0,$min,$max,1,10,0);
 	my $self = bless Gtk2::SpinButton->new($adj,10,$digits), $class;
 	$self->{noexpand}=1;
 	#$self->{field}=$field;
@@ -999,13 +1001,16 @@ sub new
 		$val=$l[0]; #take the most common value
 	}
 	else { $val=Songs::Get($IDs,$field); }
-	$self->set_value($val);
 
-	if ($specialmode)
-	{	if ($specialmode eq 'nozero')
+	if ($mode)
+	{	if ($mode eq 'nozero')		# 0 is displayed as ""
 		{	$self->signal_connect(output=> \&output_nozero);
 		}
-		elsif ($specialmode eq 'year')
+		elsif ($mode eq 'allow_empty')	# non-numeric values are replaced with "" which is treated as different than 0
+		{	$self->signal_connect(input => sub { my $v=Gtk2::Entry::get_text($_[0]); $_[0]{null}= $v!~/\d/; return 0});
+			$self->signal_connect(output=> sub { my $v=$_[0]->get_value; $_[0]{null}=0 if $v; return 0 if !$_[0]{null}; Gtk2::Entry::set_text($_[0],''); return 1; });
+		}
+		elsif ($mode eq 'year')
 		{	$self->set_wrap(1);
 			# set to current year when increasing or decreasing value from 0
 			$self->signal_connect(value_changed=>sub
@@ -1016,10 +1021,13 @@ sub new
 		}
 	}
 
+	if ($mode eq 'allow_empty' && !length $val) { $self->{null}=1; $self->set_text(''); }
+	else { $self->set_value($val); }
+
 	return $self;
 }
 sub get_text
-{	$_[0]->get_value;
+{	$_[0]{null} ? '' : $_[0]->get_value;
 }
 sub set_text
 {	my $v=$_[1];
@@ -1066,32 +1074,41 @@ sub get_text
 
 package GMB::TagEdit::Combo;
 use Gtk2;
-use base 'Gtk2::Combo';
+use base 'Gtk2::Box';
 
 sub new
 {	my ($class,$field,$IDs,$listall) = @_;
-	my $self= bless Gtk2::Combo->new, $class;
+	my $self= bless Gtk2::HBox->new, $class;
+	my $combo= Gtk2::ComboBoxEntry->new_text;
+	$self->add($combo);
+	$self->{combo}=$combo;
+	my $entry=$self->{entry}=$combo->child;
 	#$self->{field}=$field;
 
 	my $values= Songs::BuildHash($field,$IDs);
 	my @l=sort { $values->{$b} <=> $values->{$a} } keys %$values; #sort values by their frequency
 	my $first=$l[0];
 	@l= @{ Songs::Gid_to_Get($field,\@l) } if Songs::Field_property($field,'gid_to_get');
-	if ($listall) { push @l, @{Songs::ListAll($field)}; }
-	$self->set_case_sensitive(1);
-	$self->set_popdown_strings(@l);
-	$self->set_text('') unless ( $values->{$first} > @$IDs/3 );
+	if ($listall)
+	{	my $cb=sub
+		{	::PopupAA(Songs::MainField($field),noalt=>1, cb=> sub { $entry->set_text( Songs::Gid_to_Get($field,$_[0]{key}) ); });
+		};
+		my $pick= ::NewIconButton('gtk-index',undef,$cb,'none',_"Pick an existing one");
+		$self->pack_end($pick,0,0,0);
+	}
+	$combo->append_text($_) for @l;
+	$entry->set_text($l[0]) if $values->{$first} > @$IDs/3;
 
-	GMB::ListStore::Field::setcompletion($self->entry,$field) if $listall;
+	GMB::ListStore::Field::setcompletion($entry,$field) if $listall;
 
 	return $self;
 }
 
 sub set_text
-{	$_[0]->entry->set_text($_[1]);
+{	$_[0]{entry}->set_text($_[1]);
 }
 sub get_text
-{	$_[0]->entry->get_text;
+{	$_[0]{entry}->get_text;
 }
 sub tool
 {	&GMB::TagEdit::EntryString::tool;
@@ -1452,7 +1469,7 @@ sub advanced_cb
 	my $edittag=EditTag->new($dialog,$ID);
 	unless ($edittag) { ::ErrorMessage(_"Can't read file or invalid file"); return }
 	$dialog->vbox->add($edittag);
-	::SetWSize($dialog,'AdvTag');
+	::SetWSize($dialog,'AdvTag','540x505');
 	$dialog->show_all;
 	$self->{window}->set_sensitive(0);
 	$dialog->signal_connect( response => sub
