@@ -8,7 +8,7 @@
 =for gmbplugin ALBUMINFO
 name	Albuminfo
 title	Albuminfo plugin
-version	0.11
+version	0.2
 author  Øystein Tråsdahl (based on the Artistinfo plugin)
 desc	Retrieves album-relevant information (review etc.) from allmusic.com.
 =cut
@@ -26,13 +26,13 @@ use Gtk2::Gdk::Keysyms;
 use base 'Gtk2::Box';
 use constant
 {	OPT	=> 'PLUGIN_ALBUMINFO_',
+	AMG_SEARCH_URL => 'http://www.allmusic.com/search/album/',
+	AMG_ALBUM_URL => 'http://www.allmusic.com/album/',
 };
 
 my @showfields =
-(	{short => 'label',	long => _"Record label",	active => 1,	multi => 0,	defaultshow => 1},
-	{short => 'rec_date',	long => _"Recording date",	active => 1,	multi => 0,	defaultshow => 1},
+(	{short => 'rec_date',	long => _"Recording date",	active => 1,	multi => 0,	defaultshow => 1},
 	{short => 'rls_date',	long => _"Release date",	active => 1,	multi => 0,	defaultshow => 1},
-	{short => 'type',	long => _"Recording type",	active => 0,	multi => 0,	defaultshow => 0},
 	{short => 'time',	long => _"Running time",	active => 0,	multi => 0,	defaultshow => 0},
 	{short => 'rating',	long => _"Rating",		active => 1,	multi => 0,	defaultshow => 1},
 	{short => 'genre',	long => _"Genres",		active => 1,	multi => 1,	defaultshow => 0},
@@ -42,21 +42,32 @@ my @showfields =
 );
 
 ::SetDefaultOptions(OPT, PathFile  	=> "~/.config/gmusicbrowser/review/%a - %l.txt",
+			 ShowCover	=> 1,
 			 CoverSize	=> 100,
 			 StyleAsGenre	=> 0,
+			 mass_download	=> 'missing',
 );
 ::SetDefaultOptions(OPT, 'Show'.$_->{short} => $_->{defaultshow}) for (@showfields);
-
+delete $::Options{OPT.'Column'.$_} for 0..3; #remove old column options
 
 my $albuminfowidget =
 {	class		=> __PACKAGE__,
 	tabicon		=> 'plugin-albuminfo',
 	tabtitle	=> _"Albuminfo",
-	schange		=> \&song_changed,
+	schange		=> sub { $_[0]->song_changed },
 	group		=> 'Play',
 	autoadd_type	=> 'context page text',
 };
 
+my %Columns=
+(	album	=> { name=> _"Album",				storecol=>0, width=>130, },
+	artist	=> { name=> _"Artist",				storecol=>1, width=>130, },
+	genre	=> { name=> _"Genre",				storecol=>2, width=>110, },
+	year	=> { name=> _"Year",				storecol=>3, width=>50, },
+);
+
+my @towrite;              # Needed to avoid progress bar overflow in save_fields when called from mass_download
+my $save_fields_lock = 0; # Needed to avoid progress bar overflow in save_fields when called from mass_download
 
 sub Start {
 	Layout::RegisterWidget(PluginAlbuminfo => $albuminfowidget);
@@ -67,9 +78,12 @@ sub Stop {
 }
 
 sub prefbox {
-	my $spin_picsize = ::NewPrefSpinButton(OPT.'CoverSize',50,500, step=>5, page=>10, text1=>_"Cover Size : ", text2=>_"(applied after restart)");
-	my $btn_amg      = ::NewIconButton('plugin-artistinfo-allmusic',undef, sub {::main::openurl("http://www.allmusic.com/"); },'none',_"Open allmusic.com in your web browser");
-	my $hbox_picsize = ::Hpack($spin_picsize, '-', $btn_amg);
+	my $frame_cover  = Gtk2::Frame->new(' '._("Album cover").' ');
+	my $spin_picsize = ::NewPrefSpinButton(OPT.'CoverSize',50,500, step=>5, page=>10, text=>_"Cover Size : ", cb=>sub { ::HasChanged('plugin_albuminfo_option_pic'); } );
+	my $chk_picshow  = ::NewPrefCheckButton(OPT.'ShowCover'=>_"Show", widget => $spin_picsize, cb=>sub { ::HasChanged('plugin_albuminfo_option_pic'); });
+	# my $btn_amg      = ::NewIconButton('plugin-artistinfo-allmusic',undef, sub {::main::openurl("http://www.allmusic.com/"); },'none',_"Open allmusic.com in your web browser");
+	# my $hbox_picsize = ::Hpack($spin_picsize, '-', $btn_amg);
+	$frame_cover->add($chk_picshow);
 
 	my $frame_review = Gtk2::Frame->new(_" Review ");
 	my $entry_path   = ::NewPrefEntry(OPT.'PathFile' => _"Save album info in:", width=>40);
@@ -80,19 +94,19 @@ sub prefbox {
 		$t = $t ? ::PangoEsc(_("example : ").$t) : "<i>".::PangoEsc(_"invalid pattern")."</i>";
 		return '<small>'.$t.'</small>';
 	});
-	my $chk_autosave = ::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds"); #, tip=>_"Only works when the review tab is displayed");
+	my $chk_autosave = ::NewPrefCheckButton(OPT.'AutoSave'=>_"Auto-save positive finds", cb=>sub { ::HasChanged('plugin_albuminfo_option_save'); });
 	$frame_review->add(::Vpack($entry_path,$lbl_preview,$chk_autosave));
 
 	my $frame_fields = Gtk2::Frame->new(_" Fields ");
-	my $chk_join = ::NewPrefCheckButton(OPT.'StyleAsGenre' => _"Include Styles in Genres", tip=>_"Allmusic uses both Genres and Styles to describe albums. If you use only Genres, you may want to include Styles in the list of Genres.");
+	my $chk_join = ::NewPrefCheckButton(OPT.'StyleAsGenre' => _"Include Styles in Genres", tip=>_"Allmusic uses both Genres and Styles to describe albums. If you use only Genres, you may want to include Styles in allmusic's list of Genres.");
 	my @chk_fields;
 	for my $field (qw(genre mood style theme)) {
 		push(@chk_fields, ::NewPrefCheckButton(OPT.$field=>_(ucfirst($field)."s"), tip=>_"Note: inactive fields must be enabled by the user in the 'Fields' tab in Settings"));
 		$chk_fields[-1]->set_sensitive(0) unless Songs::FieldEnabled($field);
 	}
-	my ($radio_add,$radio_rpl) = ::NewPrefRadio(OPT.'ReplaceFields',undef,_"Add to existing values",1,_"Replace existing values",0);
+	my ($radio_add,$radio_rpl) = ::NewPrefRadio(OPT.'ReplaceFields',[_"Add to existing values",1, _"Replace existing values",0]);
 	my $chk_saveflds = ::NewPrefCheckButton(OPT.'SaveFields'=>_"Auto-save fields with data from allmusic", widget=>::Vpack(\@chk_fields, $radio_add, $radio_rpl),
-		tip=>_"Save selected fields for all tracks on the same album whenever album data is loaded from AMG or from file.");
+		tip=>_"Save selected fields for all tracks on the same album whenever album data is loaded from allmusic or from file.");
 	$frame_fields->add(::Vpack($chk_join, $chk_saveflds));
 
 	my $frame_layout = Gtk2::Frame->new(_" Context pane layout ");
@@ -101,7 +115,47 @@ sub prefbox {
 		push(@chk_show, ::NewPrefCheckButton(OPT.'Show'.$f->{short} => $f->{long})) if $f->{active};
 	}
 	$frame_layout->add(::Hpack(@chk_show));
-	return ::Vpack($hbox_picsize, $frame_review, $frame_fields, $frame_layout);
+
+	my $btn_download = Gtk2::Button->new(_"Download");
+	$btn_download->set_tooltip_text(_"Fields will be saved according to the settings above. Albuminfo files will be re-read if there are fields to be saved and you choose 'albums missing reviews' in the combo box.");
+	my $cmb_download = ::NewPrefCombo(OPT.'mass_download',  {all=>_"entire collection", missing=>_"albums missing reviews"}, text=>_"album information now for");
+	$btn_download->signal_connect(clicked => \&mass_download);
+	return ::Vpack($frame_cover, $frame_review, $frame_fields, $frame_layout, [$btn_download,$cmb_download]);
+}
+
+sub mass_download {
+	my $self = ::find_ancestor($_[0], __PACKAGE__);
+	$self->{aIDs} = Songs::Get_all_gids('album');
+	$self->{end} = scalar(@{$self->{aIDs}});
+	$self->{progress} = 0;
+	$self->{abort} = 0;
+	::Progress('albuminfo', end=>$self->{end}, aborthint=>_"Stop fetching albuminfo", title=>_"Fetching albuminfo",
+		abortcb=>sub {$self->{abort}=1; $self->cancel(); ::Progress('albuminfo', abort=>1)});
+	::IdleDo('9_download_one'.$self, undef, \&download_one, $self);
+}
+
+sub download_one {
+	my ($self) = @_;
+	::Progress('albuminfo', current=>$self->{progress}, bartext=>($self->{progress}+1)." / $self->{end}");
+	return if $self->{progress} >= $self->{end} || $self->{abort};
+	my $aID = $self->{aIDs}->[$self->{progress}++];
+	warn "Albuminfo: mass download in progress... $self->{progress}/$self->{end}\n" if $::debug;
+	my $IDs = Songs::MakeFilterFromGID('album',$aID)->filter();
+	my $ID  = $IDs->[0]; # Need a track (any track) from the album: pick the first in the list.
+	my $album = Songs::Get($ID, 'album');
+	unless ($album) {::IdleDo('9_download_one'.$self, undef, \&download_one, $self); return}
+	my $file = ::pathfilefromformat($ID, $::Options{OPT.'PathFile'}, undef, 1);
+	if ($::Options{OPT.'mass_download'} ne 'all' && $file && -r $file) {
+		if ($::Options{OPT.'SaveFields'}) {
+			::IdleDo('9_load_albuminfo'.$self,undef,\&load_file,$self,$ID,$file,1,\&download_one);
+		} else {
+			::IdleDo('9_download_one'.$self, undef, \&download_one, $self);
+		}
+	} else {
+		my $url = AMG_SEARCH_URL.::url_escapeall($album);
+		warn "Albuminfo: fetching search results from $url\n" if $::debug;
+		$self->{waiting} = Simple_http::get_with_cb(url=>$url, cache=>1, cb=>sub {$self->load_search_results($ID,1,\&download_one,@_)});
+	}
 }
 
 
@@ -119,8 +173,21 @@ sub new {
 	$self->{fontsize} = $fontsize->get_size() / Gtk2::Pango->scale;
 
 	# Heading: cover and album info.
-	my $cover = Layout::NewWidget("Cover", {group=>$options->{group}, forceratio=>1, maxsize=>$::Options{OPT.'CoverSize'}, xalign=>0, tip=>_"Click to show fullsize image",
-		click1=>\&cover_popup, click3=>sub {::PopupAAContextMenu({self=>$_[0], field=>'album', ID=>$::SongID, gid=>Songs::Get_gid($::SongID,'album'), mode=>'P'});} });
+	my $cover=Gtk2::HBox->new(0,0);
+	my $group=$options->{group};
+	my $cover_create= sub
+	{	my $box=shift;
+		$box->remove($_) for $box->get_children;
+		return unless $::Options{OPT.'ShowCover'};
+		my $child = Layout::NewWidget("Cover", {group=>$group, forceratio=>1, maxsize=>$::Options{OPT.'CoverSize'},
+			xalign=>0, tip=>_"Click to show larger image", click1=>\&cover_popup,
+			click3=>sub {::PopupAAContextMenu({self=>$_[0], field=>'album', ID=>$::SongID, gid=>Songs::Get_gid($::SongID,'album'), mode=>'P'});} });
+		$child->show_all;
+		$box->add($child);
+	};
+	::Watch($cover, plugin_albuminfo_option_pic=> $cover_create);
+	$cover_create->($cover);
+
 	my $statbox = Gtk2::VBox->new(0,0);
 	for my $name (qw/Ltitle Lstats/) {
 		my $l = Gtk2::Label->new('');
@@ -133,9 +200,15 @@ sub new {
 	$statbox->pack_start($self->{ratingpic},0,0,2);
 
 	# "Refresh", "save" and "search" buttons
-	my $refreshbutton = ::NewIconButton('gtk-refresh', undef, sub { song_changed(::find_ancestor($_[0],__PACKAGE__),undef,undef,1); }, "none", _"Refresh");
+	my $refreshbutton = ::NewIconButton('gtk-refresh', undef, sub { song_changed($_[0],'force'); }, "none", _"Refresh");
 	my $savebutton	  = ::NewIconButton('gtk-save', undef, sub
 		{my $self=::find_ancestor($_[0],__PACKAGE__); save_review(::GetSelID($self),$self->{fields})}, "none", _"Save review");
+	$savebutton->show_all;
+	$savebutton->set_no_show_all(1);
+	my $update_savebutton_visible= sub { $_[0]->set_visible( !$::Options{OPT.'AutoSave'} ); };
+	::Watch( $savebutton, plugin_albuminfo_option_save=> $update_savebutton_visible);
+	$update_savebutton_visible->($savebutton);
+
 	my $searchbutton = Gtk2::ToggleButton->new();
 	$searchbutton->set_relief('none');
 	$searchbutton->add(Gtk2::Image->new_from_stock('gtk-find','menu'));
@@ -143,7 +216,7 @@ sub new {
 		if ($_[0]->get_active()) {$self->manual_search()} else {$self->song_changed()}});
 	my $buttonbox = Gtk2::HBox->new();
 	$buttonbox->pack_end($searchbutton,0,0,0);
-	$buttonbox->pack_end($savebutton,0,0,0); # unless $::Options{OPT.'AutoSave'};
+	$buttonbox->pack_end($savebutton,0,0,0);
 	$buttonbox->pack_end($refreshbutton,0,0,0);
 	$statbox->pack_end($buttonbox,0,0,0);
 	my $stateventbox = Gtk2::EventBox->new(); # To catch mouse events
@@ -186,10 +259,32 @@ sub new {
 	my $Bcancel = Gtk2::Button->new_from_stock('gtk-cancel');
 	$Bok    ->set_size_request(80, -1);
 	$Bcancel->set_size_request(80, -1);
-	$self->{resultsbox}	= my $resultsbox = Gtk2::VBox->new(0,0);
-	my $scrwin  = Gtk2::ScrolledWindow->new();
+	# Year is a 'Glib::String' to avoid printing "0" when year is missing. Caveat: will give wrong sort order for albums released before year 1000 or after year 9999 :)
+	my $store = Gtk2::ListStore->new(('Glib::String')x5,'Glib::UInt'); # Album, Artist, Label, Year, URL, Sort order.
+	my $treeview = Gtk2::TreeView->new($store);
+	my %coladded;
+	for my $col ( split(/\s+/,$::Options{OPT.'Columns'}||''), qw/album artist genre year/ )
+	{	my $coldef= $Columns{$col};
+		next unless $coldef;
+		next if $coladded{$col}++; #only add a column once
+		my $colopt= $::Options{OPT.'Column_'.$col} || {};
+		my $column = Gtk2::TreeViewColumn->new_with_attributes($coldef->{name}, Gtk2::CellRendererText->new(), text=>$coldef->{storecol});
+		$column->{key}=$col;
+		$column->set_sort_column_id($coldef->{storecol}); $column->set_expand(1); $column->set_resizable(1); $column->set_reorderable(1);
+		$column->set_sizing('fixed');
+		$column->set_fixed_width( $colopt->{width}||$coldef->{width}||100 );
+		$column->set_visible(!$colopt->{hide});
+		$treeview->append_column($column);
+		# Recreate the header label to be able to catch mouse clicks in column header:
+		my $label = Gtk2::Label->new($coldef->{name}); $column->set_widget($label); $label->show;
+		my $button = $label->get_ancestor('Gtk2::Button'); # The header label is attached to a button by Gtk
+		$button->signal_connect(button_press_event => \&treeview_click_cb, $col) if $button;
+	}
+	$treeview->set_rules_hint(1);
+	$treeview->signal_connect(row_activated => \&entry_selected_cb);
+	my $scrwin = Gtk2::ScrolledWindow->new();
 	$scrwin->set_policy('automatic', 'automatic');
-	$scrwin->add_with_viewport($resultsbox);
+	$scrwin->add($treeview);
 	$searchview->add( ::Vpack(['_', $search, $Bsearch],
 				  '_',  $scrwin,
 				  '-',  ['-', $Bcancel, $Bok]) );
@@ -197,9 +292,8 @@ sub new {
 	$Bsearch->signal_connect(clicked  => \&new_search );
 	$Bok    ->signal_connect(clicked  => \&entry_selected_cb );
 	$Bcancel->signal_connect(clicked  => \&song_changed );
-	$scrwin ->signal_connect(key_press_event => sub {entry_selected_cb(::find_ancestor($_[0],__PACKAGE__)) if $_[1]->keyval == $Gtk2::Gdk::Keysyms{Return};});
-	$scrwin ->signal_connect(key_press_event => sub {song_changed(::find_ancestor($_[0],__PACKAGE__))      if $_[1]->keyval == $Gtk2::Gdk::Keysyms{Escape};});
-	$search ->signal_connect(key_press_event => sub {song_changed(::find_ancestor($_[0],__PACKAGE__))      if $_[1]->keyval == $Gtk2::Gdk::Keysyms{Escape};});
+	$scrwin ->signal_connect(key_press_event => sub {entry_selected_cb($_[0]) if $_[1]->keyval == $Gtk2::Gdk::Keysyms{Return};});
+	$scrwin ->signal_connect(key_press_event => sub {song_changed($_[0])      if $_[1]->keyval == $Gtk2::Gdk::Keysyms{Escape};});
 	$searchview->show_all(); # Must call it once now before $searchview->set_no_show_all(1) disables it.
 	$searchview->set_no_show_all(1); # GMB sometimes calls $plugin->show_all(). We then want only infoview to show.
 	$searchview->hide();
@@ -214,11 +308,56 @@ sub new {
 
 	# Save elements that will be needed in other methods.
 	$self->{buffer} = $textview->get_buffer();
+	$self->{treeview} = $treeview;
 	$self->{infoview} = $infoview;
 	$self->{searchview} = $searchview;
+	$self->{SaveOptions}= \&SaveOptions; #called when widget is removed or when saving options
 	return $self;
 }
 
+
+sub SaveOptions {
+	my $self = shift;
+	my @cols= $self->{treeview}->get_columns;
+	$::Options{OPT.'Columns'}= join ' ', map $_->{key}, @cols;
+	for my $col (@cols) {
+		my $colopt= $::Options{OPT.'Column_'.$col->{key}}= {};
+		$colopt->{width}= $col->get_width;
+		$colopt->{hide}=1 if !$col->get_visible;
+	}
+}
+
+# Called when headers in the results table in manual search are clicked
+sub treeview_click_cb {
+	my ($button, $event, $colid) = @_;
+	my $treeview = $button->parent;
+	if ($event->button == 1) {
+		my ($sortid,$order) = $treeview->get_model->get_sort_column_id();
+		my $storecol= $Columns{$colid}{storecol};
+		if ($sortid == $storecol && $order eq 'descending') {
+			$treeview->get_model->set_sort_column_id(5,'ascending'); # After third click on column header: return to AMG sort order (default).
+			return ::TRUE;
+		}
+	} elsif ($event->button == 3) {
+		my $menu=::BuildChoiceMenu( { map { ($_=>$Columns{$_}{name}) } keys %Columns }, #hash of colkey => name
+					reverse=>1,
+					args => {treeview=>$treeview},
+					check=> sub { [map $_->{key}, grep $_->get_visible, $_[0]{treeview}->get_columns]; }, #list of visible columns
+					code => sub {
+						my ($args,$key)=@_;
+						my @cols= $args->{treeview}->get_columns;
+						my ($col)= grep $_->{key} eq $key, @cols;
+						$col->set_visible( !$col->get_visible ) if $col;
+						$cols[0]->set_visible(1) unless grep $_->get_visible, @cols; #make sure one column is visible
+					},
+			);
+		$menu->show_all;
+		$menu->popup(undef,undef,undef,undef,$event->button,$event->time);
+		return ::TRUE;
+	}
+	return ::FALSE; # Let Gtk handle it
+}
+
 sub update_cursor_cb {
 	my $textview = $_[0];
 	my (undef,$wx,$wy,undef) = $textview->window->get_pointer();
@@ -245,11 +384,10 @@ sub button_release_cb {
 		if ($tag->{url}) {
 			::main::openurl($tag->{url});
 			last;
-		} elsif ($tag->{field} eq 'year') {
-			my $aID = Songs::Get_gid(::GetSelID($self),'album');
-			Songs::Set(Songs::MakeFilterFromGID('album', $aID)->filter(), [$tag->{field} => $tag->{val}]);
 		} else {
-			Songs::Set(Songs::MakeFilterFromGID('album', Songs::Get_gid(::GetSelID($self),'album'))->filter(), ['+'.$tag->{field} => $tag->{val}]);
+			my $field= $tag->{field} eq 'year' ? 'year' : '+'.$tag->{field}; # prepend + for multi-value fields : Genre, Mood, Style, Theme
+			my $aID = Songs::Get_gid(::GetSelID($self),'album');
+			Songs::Set(Songs::MakeFilterFromGID('album', $aID)->filter(), [$field => $tag->{val}]);
 		}
 	}
 	return ::FALSE;
@@ -259,9 +397,10 @@ sub cover_popup {
 	my ($self, $event) = @_;
 	my $menu = Gtk2::Menu->new();
 	$menu->modify_bg('GTK_STATE_NORMAL',Gtk2::Gdk::Color->parse('black')); # black bg for the cover-popup
+	my $picsize = 400;
 	my $ID = ::GetSelID($self);
 	my $aID = Songs::Get_gid($ID,'album');
-	if (my $img = Gtk2::Image->new_from_file(AAPicture::GetPicture('album',$aID))) {
+	if (my $img = AAPicture::newimg(album=>$aID,$picsize)) {
 		my $apic = Gtk2::MenuItem->new();
 		$apic->modify_bg('GTK_STATE_SELECTED',Gtk2::Gdk::Color->parse('black'));
 		$apic->add($img);
@@ -343,9 +482,10 @@ sub print_review {
 	} else {
 		$buffer->insert_with_tags($iter,"\n"._("No review written.")."\n",$tag_h2);
 	}
+	$buffer->insert($iter, "\n\n");
 	my $tag_a  = $buffer->create_tag(undef, foreground=>"#4ba3d2", underline=>'single');
 	$tag_a->{url} = $fields->{url}; $tag_a->{tip} = $fields->{url};
-	$buffer->insert_with_tags($iter,"\n\n"._"Lookup at allmusic.com",$tag_a);
+	$buffer->insert_with_tags($iter,_"Lookup at allmusic.com",$tag_a);
 	$buffer->set_modified(0);
 }
 
@@ -361,7 +501,6 @@ sub manual_search {
 	$self->{infoview}->hide();
 	$self->{searchview}->show();
 	my $gid = Songs::Get_gid(::GetSelID($self), 'album');
-	my $album = Songs::Gid_to_Get("album",$gid);
 	$self->{search}->set_text(Songs::Gid_to_Get('album', $gid));
 	$self->new_search();
 }
@@ -371,64 +510,51 @@ sub new_search {
 	my $album = $self->{search}->get_text();
 	$album =~ s|^\s+||; $album =~ s|\s+$||; # remove leading and trailing spaces
 	return if $album eq '';
-	$self->{resultsbox}->remove($_) for $self->{resultsbox}->get_children();
-	$self->{resultsbox}->pack_start(Gtk2::Label->new('Loading...'),0,0,20);
-	$self->{resultsbox}->show_all();
-	my $url = "http://allmusic.com/search/album/".::url_escapeall($album);
+	my $url = AMG_SEARCH_URL.::url_escapeall($album);
 	$self->cancel();
-	warn "Albuminfo: fetching AMG search from url $url.\n" if $::debug;
+	$self->{treeview}->get_model->clear;
+	warn "Albuminfo: fetching search results from $url.\n" if $::debug;
 	$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->print_results(@_)},url=>$url, cache=>1);
 }
 
 sub print_results {
 	my ($self,$html,$type,$url) = @_;
 	delete $self->{waiting};
-	$self->{resultsbox}->remove($_) for $self->{resultsbox}->get_children();
 	my $result = parse_amg_search_results($html, $type); # result is a ref to an array of hash refs
-	my @radios;
-	if ( $#{$result} + 1 ) {
-		push(@radios, Gtk2::RadioButton->new(undef, "$_->{artist} - $_->{album} ($_->{year}) on $_->{label}")) for @$result;
-		my $group = $radios[0]->get_group();
-		$radios[$_]->set_group($group) for (1 .. $#radios);
-		$self->{resultsbox}->pack_start($_,0,0,0) for @radios;
-	} else {
-		$self->{resultsbox}->pack_start(Gtk2::Label->new(_"No results found."),0,0,20);
+	my $store= $self->{treeview}->get_model;
+	$store->set_sort_column_id(5, 'ascending');
+	for (@$result) {
+		$store->set($store->append, 0,$_->{album}, 1,$_->{artist}, 2,$_->{genre}, 3,$_->{year}, 4,$_->{url}, 5,$_->{order});
 	}
-	$self->{result} = $result;
-	$self->{radios} = \@radios;
-	$self->{resultsbox}->show_all();
 }
 
 sub entry_selected_cb {
-	my $self = ::find_ancestor($_[0], __PACKAGE__); # $_[0] is the 'OK' button. Ancestor is an albuminfo object.
-	$self->{searchview}->hide();
-	$self->{infoview}->show();
-	return unless ( $#{$self->{result}} + 1 );
-	my $selected;
-	for (0 .. $#{$self->{radios}}) {
-		if (${$self->{radios}}[$_]->get_active()) {$selected = ${$self->{result}}[$_]; last;}
-	}
-	warn "Albuminfo: fetching review from url $selected->{url}\n" if $::debug;
-	$self->{url} = $selected->{url}.'/review';
+	my $self = ::find_ancestor($_[0], __PACKAGE__); # $_[0] may be the TreeView or the 'OK' button. Ancestor is an albuminfo object.
+	my ($path, $column) = $self->{treeview}->get_cursor();
+	unless (defined $path) {$self->{searchview}->hide(); $self->{infoview}->show(); return} # The user may click OK before selecting an album
+	my $store = $self->{treeview}->get_model;
+	my $url = $store->get($store->get_iter($path),4);
+	warn "Albuminfo: fetching review from $url\n" if $::debug;
 	$self->cancel();
-	$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->load_review(::GetSelID($self),@_)}, url=>$self->{url}, cache=>1);
+	$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->{searchview}->hide(); $self->{infoview}->show();
+		$self->load_review(::GetSelID($self),0,undef,$url,@_)}, url=>$url, cache=>1);
 }
 
 
 
-
 #####################################################################
 #
 # Section: loading and parsing review. Saving review and fields.
 #
 #####################################################################
 sub song_changed {
-	my ($widget,$tmp,$group,$force) = @_; # $tmp = ::GetSelID($self), but not always. So we don't use it. $group is also not used.
+	my ($widget,$force) = @_;
 	my $self = ::find_ancestor($widget, __PACKAGE__);
 	return unless $self->mapped() || $::Options{OPT.'SaveFields'};
 	$self->{infoview}->show();
 	$self->{searchview}->hide();
 	my $ID = ::GetSelID($self);
+	return unless defined $ID;
 	my $aID = Songs::Get_gid($ID,'album');
 	return unless $aID;
 	if (!$self->{aID} || $aID != $self->{aID} || $force) { # Check if album has changed or a forced update is required.
@@ -446,17 +572,17 @@ sub album_changed {
 	$self->update_titlebox($aID);
 	my $album = ::url_escapeall(Songs::Gid_to_Get("album",$aID));
 	unless (length($album)) {$self->print_warning(_"Unknown album"); return}
-	my $url = "http://allmusic.com/search/album/$album";
+	my $url = AMG_SEARCH_URL.$album;
 	$self->print_warning(_"Loading...");
 	unless ($force) { # Try loading from file.
 		my $file = ::pathfilefromformat( ::GetSelID($self), $::Options{OPT.'PathFile'}, undef, 1 );
 		if ($file && -r $file) {
-			::IdleDo('9_load_albuminfo'.$self,undef,\&load_file,$self,$ID,$file);
+			::IdleDo('9_load_albuminfo'.$self,undef,\&load_file,$self,$ID,$file,0,undef);
 			return;
 		}
 	}
-	warn "Albuminfo: fetching search results from url $url\n" if $::debug;
-	$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->load_search_results($ID,@_)}, url=>$url, cache=>1);
+	warn "Albuminfo: fetching search results from $url\n" if $::debug;
+	$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->load_search_results($ID,0,undef,@_)}, url=>$url, cache=>1);
 }
 
 sub update_titlebox {
@@ -475,40 +601,40 @@ sub update_titlebox {
 }
 
 sub load_search_results {
-	my ($self,$ID,$html,$type) = @_;
+	my ($self,$ID,$md,$cb,$html,$type) = @_; # $md = 1 if mass_download, 0 otherwise. $cb = callback function if mass_download, undef otherwise.
 	delete $self->{waiting};
-	my $result = parse_amg_search_results($html, $type);
+	my $result = parse_amg_search_results($html, $type); # $result[$i] = {url, album, artist, genre, year}
 	my ($artist,$year) = ::Songs::Get($ID, qw/artist year/);
 	my $url;
 	for my $entry (@$result) {
 		# Pick the first entry with the right artist and year, or if not: just the right artist.
-		# if (::superlc($entry->{artist}) eq ::superlc($artist)) {
-		if ($entry->{artist} =~ m|$artist|i || $artist =~ m|$entry->{artist}|i) {
-			if (!$url || $entry->{year} == $year) {
-				warn "Albuminfo: AMG hit: $entry->{album} by $entry->{artist} from year=$entry->{year} ($entry->{url})\n" if $::debug;
-				$url = $entry->{url}."/review";
+		if (::superlc($entry->{artist}) eq ::superlc($artist)) {
+			if (!$url || ($entry->{year} && $entry->{year} == $year)) {
+				warn "Albuminfo: hit in search results: $entry->{album} by $entry->{artist} from $entry->{year} ($entry->{url})\n" if $::debug;
+				$url = $entry->{url};
 			}
 			last if $year && $entry->{year} && $entry->{year} == $year;
 		}
 	}
 	if ($url) {
-		$self->{url} = $url;
-		warn "Albuminfo: fetching review from url $url\n" if $::debug;
-		$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->load_review($ID,@_)}, url=>$url, cache=>1);
+		warn "Albuminfo: fetching review from $url\n" if $::debug;
+		$self->{waiting} = Simple_http::get_with_cb(cb=>sub {$self->load_review($ID,$md,$cb,$url,@_)}, url=>$url, cache=>1);
 	} else {
 		$self->{fields} = {};
-		$self->print_warning(_"No review found");
-	}
+		warn "Albuminfo: album not found in search results\n" if $::debug;
+		$self->print_warning(_"No review found") unless $md;
+		::IdleDo('9_download_one'.$self, undef, $cb, $self) if $cb;
+	}	
 }
 
 sub load_review {
-	my ($self,$ID,$html,$type) = @_;
+	my ($self,$ID,$md,$cb,$url,$html,$type) = @_;
 	delete $self->{waiting};
-	$self->{fields} = parse_amg_album_page($html,$type);
-	$self->{fields}{url} = $self->{url}; # So that url gets stored if AutoSave is chosen.
-	$self->print_review();
-	save_review($ID, $self->{fields}) if $::Options{OPT.'AutoSave'};
-	save_fields($ID, $self->{fields}) if $::Options{OPT.'SaveFields'};
+	$self->{fields} = parse_amg_album_page($url,$html,$type);
+	$self->print_review() unless $md;
+	save_review($ID, $self->{fields}) if $::Options{OPT.'AutoSave'} || $md;
+	if ($::Options{OPT.'SaveFields'}) {push(@towrite, [$ID, %{$self->{fields}}]); save_fields()}
+	::IdleDo('9_download_one'.$self, undef, $cb, $self) if $cb;
 }
 
 sub parse_amg_search_results {
@@ -516,40 +642,48 @@ sub parse_amg_search_results {
 	$html = decode($html, $type);
 	$html =~ s/\n/ /g;
 	# Parsing the html yields @res = (url1, album1, artist1, label1, year1, url2, album2, ...)
-	my @res = $html =~ m|<a href="(http://allmusic\.com/album.*?)">(.*?)</a></td>\s.*?<td>(.*?)</td>\s.*?<td>(.*?)</td>\s.*?<td>(.*?)</td>|g;
-	my @fields = qw/url album artist label year/;
+	#my @res = $html =~ m|<a href="(http://www\.allmusic\.com/album.*?)">(.*?)</a></td>\s.*?<td>(.*?)</td>\s.*?<td>(.*?)</td>\s.*?<td>(.*?)</td>|g;
+	my @res = $html =~ m|<a href="(http://www\.allmusic\.com/album.*?)" data-tooltip=".*?">(.*?)</a>.*?class="artist">.*?<a href=".*?">(.*?)</a>.*?"info">\s*?(\S*?)\s*?<br/>\s*?(\S.*?)\s*?</div>|g;
+	#my @fields = qw/url album artist label year/;
+	my @fields = qw/url album artist year genre/;
 	my @result;
-	push(@result, {%_}) while (@_{@fields} = splice(@res, 0, 5)); # create an array of hash refs
+	my $i = 0; # Used to sort the hits in manual search
+	while (@res)
+	{	my %hash= (order=>$i++);
+		@hash{@fields}= splice @res, 0, 5;
+		push @result, \%hash; # create an array of hash refs
+	}
 	return \@result;
 }
 
 sub parse_amg_album_page {
-	my ($html,$type) = @_;
+	my ($url,$html,$type) = @_;
 	$html = decode($html, $type);
 	$html =~ s|\n| |g;
 	my $result = {};
-	$result->{author} = $1 if $html =~ m|<p class="author">by (.*?)</p>|;
-	$result->{review} = $1 if $html =~ m|<p class="text">(.*?)</p>|;
+	$result->{url} = $url;
+	$result->{author} = $1 if $html =~ m|class="author">by (.*?)</span>|;
+	$result->{review} = $1 if $html =~ m|<div class="editorial-text.*?>\s*<p>(.*?)</div>\s*</div>\s*</div>|;
 	if ($result->{review}) {
 		$result->{review} =~ s|<br\s.*?/>|\n|gi;      # Replace newline tags by newlines.
-		$result->{review} =~ s|<p\s.*?/>|\n\n|gi;     # Replace paragraph tags by newlines.
+		$result->{review} =~ s|<p>(.*?)</p>|$1\n\n|gi;     # Replace paragraph tags by newlines.
 		$result->{review} =~ s|\n{3,}|\n\n|gi;	      # Never more than one empty line.
-		$result->{review} =~ s|<.*?>(.*?)</.*?>|$1|g; # Remove the rest of the html tags.
+		$result->{review} =~ s|<div.*?>||g;
+		$result->{review} =~ s|</div>||g;
+$result->{review} =~ s|<.*?>(.*?)</.*?>|$1|g; # Remove the rest of the html tags.
 	}
-	$result->{rls_date}	= $1 if $html =~ m|<h3>Release Date</h3>\s*?<p>(.*?)</p>|;
-	$result->{rec_date}	= $1 if $html =~ m|<h3>Recording Date</h3>\s*?<p>(.*?)</p>|;
-	$result->{label}	= $1 if $html =~ m|<h3>Label</h3>\s*?<p>(.*?)</p>|;
-	$result->{type}		= $1 if $html =~ m|<h3>Type</h3>\s*?<p>(.*?)</p>|;
-	$result->{time}		= $1 if $html =~ m|<h3>Time</h3>\s*?<p>(.*?)</p>|;
-	$result->{amgid}	= $1 if $html =~ m|<p class="amgid">(.*?)</p>|; $result->{amgid} =~ s/R\s*/r/;
-	$result->{rating}	= 10*($1+1) if $html =~ m|star_rating\((\d+?)\)|;
-	my ($genrehtml)		= $html =~ m|<h3>Genre</h3>\s*?<ul.*?>(.*?)</ul>|;
-	(@{$result->{genre}})	= $genrehtml =~ m|<li><a.*?>\s.*?(.*?)</a></li>|g if $genrehtml;
-	my ($stylehtml)		= $html =~ m|<h3>Style</h3>\s*?<ul.*?>(.*?)</ul>|;
+	$result->{rls_date}	= $1 if $html =~ m|class="release-date">(.*?)</|;
+	$result->{rec_date}	= $1 if $html =~ m|class="record-date">(.*?)</|;
+	$result->{time}		= $1 if $html =~ m|class="duration">\s*?(.*?)\s|;
+	$result->{amgid}	= $1 if $html =~ m|AMG Pop ID.*?R\s*(.*?)</p>|;
+	$result->{rating}	= 20*$1 if $html =~ m|data-stars="(.*?)"|;
+	my ($genrehtml)		= $html =~ m|class="genres".*?<ul>(.*?)</ul>|;
+	(@{$result->{genre}})	= $genrehtml =~ m|<li><a.*?>(.*?)</a></li>|g if $genrehtml;
+	my ($stylehtml)		= $html =~ m|class="styles".*?<ul>(.*?)</ul>|;
 	(@{$result->{style}})	= $stylehtml =~ m|<li><a.*?>(.*?)</a></li>|g if $stylehtml;
-	my ($moodshtml)		= $html =~ m|<h3>Moods</h3>\s*?<ul.*?>(.*?)</ul>|;
+	my ($moodshtml)		= $html =~ m|class="sidebar-module moods">.*?<ul>(.*?)</ul>|;
 	(@{$result->{mood}})	= $moodshtml =~ m|<li><a.*?>(.*?)</a></li>|g if $moodshtml;
-	my ($themeshtml)	= $html =~ m|<h3>Themes</h3>\s*?<ul.*?>(.*?)</ul>|;
+	my ($themeshtml)	= $html =~ m|class="sidebar-module themes".*?<ul.*?>(.*?)</ul>|;
 	(@{$result->{theme}})	= $themeshtml =~ m|<li><a.*?>(.*?)</a></li>|g if $themeshtml;
 	if ($::Options{OPT.'StyleAsGenre'} && $result->{style}) {@{$result->{genre}} = ::uniq(@{$result->{style}}, @{$result->{genre}})}
 	return $result;
@@ -571,10 +705,10 @@ sub decode {
 
 # Load review from file
 sub load_file {
-	my ($self,$ID,$file) = @_;
-	warn "Albuminfo: loading review from file $file\n" if $::debug;
+	my ($self,$ID,$file,$md,$cb) = @_;
 	$self->{fields} = {};
 	if ( open(my$fh, '<', $file) ) {
+		warn "Albuminfo: loading review from file $file\n" if $::debug;
 		local $/ = undef; #slurp mode
 		my $text = <$fh>;
 		if (my $utf8 = Encode::decode_utf8($text)) {$text = $utf8}
@@ -589,17 +723,16 @@ sub load_file {
 			}
 		}
 		close $fh;
+		if ($::Options{OPT.'StyleAsGenre'} && $self->{fields}->{style}) {@{$self->{fields}->{genre}} = ::uniq(@{$self->{fields}->{style}}, @{$self->{fields}->{genre}})}
+		$self->print_review() unless $md;
+		if ($::Options{OPT.'SaveFields'}) {push(@towrite, [$ID, %{$self->{fields}}]); save_fields()} # We may not have saved when first downloaded.
 	} else {
 		warn "Albuminfo: failed retrieving info from $file\n" if $::debug;
-		$self->print_warning(_"No review found");
-		$self->fields = {};
+		$self->print_warning(_"No review found") unless $md;
 	}
-	if ($::Options{OPT.'StyleAsGenre'} && $self->{fields}->{style}) {@{$self->{fields}->{genre}} = ::uniq(@{$self->{fields}->{style}}, @{$self->{fields}->{genre}})}
-	$self->print_review();
-	save_fields($ID, $self->{fields}) if $::Options{OPT.'SaveFields'}; # We may not have saved when first downloaded.
+	::IdleDo('9_download_one'.$self, undef, $cb, $self) if $cb;
 }
 
-
 # Save review to file. The format of the file is: <field>values</field>
 sub save_review {
 	my ($ID,$fields) = @_;
@@ -626,19 +759,28 @@ sub save_review {
 
 # Save selected fields (moods, styles etc.) for all tracks in album
 sub save_fields {
-	my ($ID,$fields) = @_;
+	return if $save_fields_lock;
+	return unless scalar(@towrite);
+	my ($ID,%fields) = @{shift(@towrite)};
+	my $album = Songs::Gid_to_Get("album", Songs::Get_gid($ID,'album'));
+	warn "Albuminfo: Saving tracks on $album (".scalar(@towrite)." album".(scalar(@towrite)!=1 ? "s" : "")." in queue).\n" if $::debug;
 	my $IDs = Songs::MakeFilterFromGID('album', Songs::Get_gid($ID,'album'))->filter(); # Songs on this album
 	my @updated_fields;
-	for my $key (keys %{$fields}) {
-		if ($key =~ m/genre|mood|style|theme/ && $::Options{OPT.$key} && $fields->{$key}) {
+	for my $key (qw/genre mood style theme/) {
+		if ($::Options{OPT.$key} && $fields{$key}) {
 			if ( $::Options{OPT.'ReplaceFields'} ) {
-				push(@updated_fields, $key, $fields->{$key});
+				push(@updated_fields, $key, $fields{$key});
 			} else {
-				push(@updated_fields, '+'.$key, $fields->{$key});
+				push(@updated_fields, '+'.$key, $fields{$key});
 			}
 		}
 	}
-	Songs::Set($IDs, \@updated_fields) if @updated_fields;
+	if (@updated_fields) {
+		$save_fields_lock = 1;
+		Songs::Set($IDs, \@updated_fields, callback_finish=>sub {$save_fields_lock = 0; save_fields();});
+	} else {
+		save_fields(); # There may still be albums in @towrite
+	}
 }
 
 # Cancel pending tasks, and abort possible http_wget in progress.
