@@ -69,6 +69,7 @@ my @similarity=
 
 ::SetDefaultOptions(OPT,PathFile	=> "~/.config/gmusicbrowser/bio/%a",
 			ArtistPicSize	=> 70,
+			ArtistPicShow	=> 1,
 			SimilarLimit	=> 15,
 			SimilarRating	=> 20,
 			SimilarLocal	=> 0,
@@ -82,7 +83,7 @@ my $artistinfowidget=
 	tabicon		=> 'plugin-artistinfo',		# no icon by that name by default (yet)
 	tabtitle	=> _"Artistinfo",
 	saveoptions	=> 'site',
-	schange		=> \&SongChanged,
+	schange		=> sub { $_[0]->SongChanged; },
 	group		=> 'Play',
 	autoadd_type	=> 'context page text',
 };
@@ -109,7 +110,7 @@ sub new
 	$self->{fontsize} = $fontsize->get_size / Gtk2::Pango->scale;
 	$self->{artist_esc} = "";
 	my $statbox=Gtk2::VBox->new(0,0);
-	my $artistpic = Layout::NewWidget("ArtistPic",{forceratio=>1,minsize=>$::Options{OPT.'ArtistPicSize'},click1=>\&apiczoom,xalign=>0,group=>$options->{group},tip=>_"Click to show fullsize image"});
+	my $artistpic=Gtk2::HBox->new(0,0);
 	for my $name (qw/Ltitle Lstats/)
 	{	my $l=Gtk2::Label->new('');
 		$self->{$name}=$l;
@@ -128,6 +129,18 @@ sub new
 	$artistbox->pack_start($artistpic,0,1,0);
 	$artistbox->pack_start($stateventbox,1,1,0);
 
+	my $group= $options->{group};
+	my $artistpic_create= sub
+	{	my $box=shift;
+		$box->remove($_) for $box->get_children;
+		return unless $::Options{OPT.'ArtistPicShow'};
+		my $child = Layout::NewWidget("ArtistPic",{forceratio=>1,minsize=>$::Options{OPT.'ArtistPicSize'},click1=>\&apiczoom,xalign=>0,group=>$group,tip=>_"Click to show fullsize image"});
+		$child->show_all;
+		$box->add($child);
+	};
+	::Watch($artistpic, plugin_artistinfo_option_pic=> $artistpic_create);
+	$artistpic_create->($artistpic);
+
 	my $textview=Gtk2::TextView->new;
 	$self->signal_connect(map => \&SongChanged);
 	$textview->set_cursor_visible(0);
@@ -165,17 +178,17 @@ sub new
 	$toolbar->set_style( $options->{ToolbarStyle}||'both-horiz' );
 	$toolbar->set_icon_size( $options->{ToolbarSize}||'small-toolbar' );
 	#$toolbar->set_show_arrow(1);
-	my $group; my $menugroup;
+	my $radiogroup; my $menugroup;
 	foreach my $key (sort keys %sites)
 	{	my $item = $sites{$key}[1];
-		$item = Gtk2::RadioButton->new($group,$item);
+		$item = Gtk2::RadioButton->new($radiogroup,$item);
 		$item->{key} = $key;
 		$item -> set_mode(0); # display as togglebutton
 		$item -> set_relief("none");
 		$item -> set_tooltip_text($sites{$key}[2]);
 		$item->set_active( $key eq $self->{site} );
-		$item->signal_connect('toggled' => sub { my $self=::find_ancestor($_[0],__PACKAGE__); toggled_cb($self,$item,$textview); } );
-		$group = $item -> get_group;
+		$item->signal_connect(toggled => sub { my $self=::find_ancestor($_[0],__PACKAGE__); toggled_cb($self,$item,$textview); } );
+		$radiogroup = $item -> get_group;
 		my $toolitem=Gtk2::ToolItem->new;
 		$toolitem->add( $item );
 		$toolitem->set_expand(1);
@@ -192,17 +205,24 @@ sub new
 #		$toolitem->set_proxy_menu_item($key,$menuitem);
 	}
 	for my $button
-	(	[refresh => 'gtk-refresh', sub { my $self=::find_ancestor($_[0],__PACKAGE__); SongChanged($self,'1'); },_"Refresh", _"Refresh",0],
+	(	[refresh => 'gtk-refresh', sub { my $self=::find_ancestor($_[0],__PACKAGE__); SongChanged($self,'force'); },_"Refresh", _"Refresh"],
 		[save => 'gtk-save',	\&Save_text,	_"Save",	_"Save artist biography",$::Options{OPT.'AutoSave'}],
 	)
-	{	my ($key,$stock,$cb,$label,$tip,$hide)=@$button;
+	{	my ($key,$stock,$cb,$label,$tip)=@$button;
 		my $item=Gtk2::ToolButton->new_from_stock($stock);
 		$item->signal_connect(clicked => $cb);
 		$item->set_tooltip_text($tip) if $tip;
 		my $menuitem = Gtk2::ImageMenuItem->new ($label);
 		$menuitem->set_image( Gtk2::Image->new_from_stock($stock,'menu') );
 		$item->set_proxy_menu_item($key,$menuitem);
-		$toolbar->insert($item,-1) unless $hide;
+		$toolbar->insert($item,-1);
+		if ($key eq 'save')
+		{	$item->show_all;
+			$item->set_no_show_all(1);
+			my $update= sub { $_[0]->set_visible(!$::Options{OPT.'AutoSave'}); };
+			::Watch($item, plugin_artistinfo_option_save=> $update);
+			$update->($item);
+		}
 	}
 	my $artistinfobox = Gtk2::VBox->new(0,0);
 	$artistinfobox->pack_start($artistbox,1,1,0);
@@ -258,8 +278,10 @@ sub prefbox
 {	my $vbox=Gtk2::VBox->new(0,2);
 	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Load/Save Artist Info in :", width=>50);
 	my $preview= Label::Preview->new(preview => \&filename_preview, event => 'CurSong Option', noescape=>1,wrap=>1);
-	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds", tip=>_"only works when the artist-info tab is displayed");
-	my $picsize=::NewPrefSpinButton(OPT.'ArtistPicSize',50,500, step=>5, page=>10, text =>_("Artist Picture Size : %d")._"(applied after restart)");
+	my $autosave = ::NewPrefCheckButton(OPT.'AutoSave'=>_"Auto-save positive finds", tip=>_"only works when the artist-info tab is displayed",
+		cb=>sub { ::HasChanged('plugin_artistinfo_option_save'); });
+	my $picsize=::NewPrefSpinButton(OPT.'ArtistPicSize',50,500, step=>5, page=>10, text =>_("Artist picture size : %d"), cb=>sub { ::HasChanged('plugin_artistinfo_option_pic'); });
+	my $picshow=::NewPrefCheckButton(OPT.'ArtistPicShow' => _"Show artist picture", widget => ::Vpack($picsize), cb=>sub { ::HasChanged('plugin_artistinfo_option_pic'); } );
 	my $eventformat=::NewPrefEntry(OPT.'Eventformat' => _"Enter custom event string :", expand=>1, tip => _"Use tags from last.fm's XML event pages with a leading % (e.g. %headliner), furthermore linebreaks '<br>' and any text you'd like to have in between. E.g. '%title taking place at %startDate<br>in %city, %country<br><br>'", history=>OPT.'Eventformat_history');
 	my $eventformat_reset=Gtk2::Button->new(_"reset format");
 	$eventformat_reset->{format_combo}=$eventformat;
@@ -276,7 +298,7 @@ sub prefbox
 	my $similar_exclude_seed=::NewPrefCheckButton(OPT.'SimilarExcludeSeed' => _"Exclude 'seed'-artist from queue", tip=>_"The artists similar to the 'seed'-artist will be used to populate the queue, but you can decide to exclude the 'seed'-artist him/herself.");
 	my $lastfm=::NewIconButton('plugin-artistinfo-lastfm',undef,sub { ::main::openurl("http://www.last.fm/music/"); },'none',_"Open last.fm website in your browser");
 	my $titlebox=Gtk2::HBox->new(0,0);
-	$titlebox->pack_start($picsize,1,1,0);
+	$titlebox->pack_start($picshow,1,1,0);
 	$titlebox->pack_start($lastfm,0,0,5);
 	my $frame_bio=Gtk2::Frame->new(_"Biography");
 	$frame_bio->add(::Vpack($entry,$preview,$autosave));
@@ -357,32 +379,34 @@ sub CreateSearchMenu {
 
 sub apiczoom {
 	my ($self, $event) = @_;
-	my $menu=Gtk2::Menu->new;
-	$menu->modify_bg('GTK_STATE_NORMAL',Gtk2::Gdk::Color->parse('black')); # black bg for the artistpic-popup
-	my $picsize=250;
 	my $ID = ::GetSelID($self);
 	my $aID = Songs::Get_gid($ID,'artist');
-	if (my $img= AAPicture::newimg(artist=>$aID,$picsize)) {
-		my $apic = Gtk2::MenuItem->new;
-		$apic->modify_bg('GTK_STATE_SELECTED',Gtk2::Gdk::Color->parse('black'));
-		$apic->add($img);
-		$apic->show_all;
-		my $artist = Songs::Gid_to_Get("artist",$aID);
-		my $item=Gtk2::MenuItem->new;
-		$item->modify_fg('GTK_STATE_SELECTED',Gtk2::Gdk::Color->parse('white'));
-		my $label=Gtk2::Label->new;	# use a label instead of a normal menu-item for formatted text
-		$label->set_line_wrap(1);
-		$label->set_justify('center');
-		$label->set_ellipsize('end');
-		$label->set_markup( "<big><b>$artist</b></big>" );
-		$item->add($label);
-		$item->show_all;
-		$menu->append($apic);
-		$menu->append($item);
-		$menu->popup (undef, undef, undef, undef, $event->button, $event->time);
-		return 1;
-	}
-	else { return 0; }
+	my $picsize=250;
+	my $img= AAPicture::newimg(artist=>$aID,$picsize);
+	return 0 unless $img;
+
+	my $menu=Gtk2::Menu->new;
+	$menu->modify_bg('normal',Gtk2::Gdk::Color->parse('black')); # black bg for the artistpic-popup
+	my $apic = Gtk2::MenuItem->new;
+	$apic->modify_bg('selected',Gtk2::Gdk::Color->parse('black'));
+	$apic->add($img);
+
+	my $artist = Songs::Gid_to_Get("artist",$aID);
+	my $item=Gtk2::MenuItem->new;
+	my $label=Gtk2::Label->new;	# use a label instead of a normal menu-item for formatted text
+	$item->modify_bg('selected',Gtk2::Gdk::Color->parse('black'));
+	$label->modify_fg($_,Gtk2::Gdk::Color->parse('white')) for qw/normal prelight/;
+	$label->set_line_wrap(1);
+	$label->set_justify('center');
+	$label->set_ellipsize('end');
+	$label->set_markup( "<big><b>$artist</b></big>" );
+	$item->add($label);
+
+	$menu->append($apic);
+	$menu->append($item);
+	$menu->show_all;
+	$menu->popup (undef, undef, undef, undef, $event->button, $event->time);
+	return 1;
 }
 
 sub update_cursor_cb
@@ -427,7 +451,7 @@ sub SongChanged
 {	my ($widget,$force) = @_;
 	my $self=::find_ancestor($widget,__PACKAGE__);
 	my $ID = ::GetSelID($self);
-	$force = 0 unless $force;
+	return unless defined $ID;
 	$self -> ArtistChanged( Songs::Get_gid($ID,'artist'),Songs::Get_gid($ID,'album'),$force);
 }
 
@@ -435,7 +459,6 @@ sub ArtistChanged
 {	my ($self,$aID,$albumID,$force)=@_;
 	return unless $self->mapped;
 	return unless defined $aID;
-	$self->cancel;
 	my $rating = AA::Get("rating:average",'artist',$aID);
 	$self->{artistratingvalue}= int($rating+0.5);
 	$self->{artistratingrange}=AA::Get("rating:range",'artist',$aID);
@@ -453,10 +476,11 @@ sub ArtistChanged
 
 	my $url = GetUrl($sites{$self->{site}}[SITEURL],$aID);
 	return unless $url;
-	if (!$self->{url} or $url ne $self->{url} or $force == 1) {
+	if (!$self->{url} or ($url ne $self->{url}) or $force) {
+		$self->cancel;
 		$self->{url} = $url;
 		if ($self->{site} eq "biography") { # check for local biography file before loading the page
-			unless ($force == 1) {
+			unless ($force) {
 			my $file=::pathfilefromformat( ::GetSelID($self), $::Options{OPT.'PathFile'}, undef,1 );
 			if ($file && -r $file)
 				{	::IdleDo('8_artistinfo'.$self,1000,\&load_file,$self,$file);
@@ -474,7 +498,9 @@ sub GetUrl
 	my $artist = ::url_escapeall( Songs::Gid_to_Get("artist",$aID) );
 	return unless length $artist;
 	$url=~s/%a/$artist/;
-	$url=~s/%l/$::Options{OPT.'SimilarLimit'}/;
+	my $limit = "";
+	if ($::Options{OPT.'SimilarLimit'} != "0") { $limit = $::Options{OPT.'SimilarLimit'}; }
+	$url=~s/%l/$limit/;
 	return $url;
 }
 
@@ -510,35 +536,31 @@ sub loaded
 	my $tag_extra = $buffer->create_tag(undef,foreground_gdk=>$self->style->text_aa("normal"),justification=>'left');
 	my $tag_noresults=$buffer->create_tag(undef,justification=>'center',font=>$fontsize*2,foreground_gdk=>$self->style->text_aa("normal"));
 	my $tag_header = $buffer->create_tag(undef,justification=>'left',font=>$fontsize+1,weight=>Gtk2::Pango::PANGO_WEIGHT_BOLD);
-	my ($artistinfo_ok,$infoheader,$warning);
+	my $infoheader;
 
 	if ($self->{site} eq "biography") {
 		$infoheader = _"Artist Biography";
-		$data =~ m|^.*<name>(.*?)</name>.*<url>(.*?)</url>.*<listeners>(.*?)</listeners>.*<playcount>(.*?)</playcount>.*<content><\!\[CDATA\[(.*?)\n.*\]\]></content>|s; # last part of the regexp removes the license-notice (=last line)
-		my $lfm_artist = $1;
-		my $aID = Songs::Get_gid($::SongID,'artist');
-		my $local_artist = Songs::Gid_to_Get("artist",$aID);
-		if ($lfm_artist ne $local_artist) { $warning = "Redirected to: ".$lfm_artist."\n"; }
-		else { $warning = ""; }
-		my $url = $2.'/+wiki/edit';
-		my $href = $buffer->create_tag(undef,justification=>'left',foreground=>"#4ba3d2",underline=>'single');
-		$href->{url}=$url;
-		my $listeners = $3;
-		my $playcount = $4;
-		$data = $5;
-		for ($data) {
-			s/<br \/>|<\/p>/\n/gi; # never more than one empty line
-			s/\n\n/\n/gi; # never more than one empty line (again)
-			s/<(.*?)>//gi; # strip tags
-		}
-		if ($data eq "") {
+		(my($lfm_artist,$url,$listeners,$playcount),$data)=
+			$data =~ m|^.*?<name>([^<]*)</name>.*?<url>([^<]*)</url>.*?<listeners>([^<]*)</listeners>.*?<playcount>([^<]*)</playcount>.*?<content>\W*<\!\[CDATA\[(.*?)\n.*\]\]>\W*</content>|s; # last part of the regexp removes the license-notice (=last line)
+
+		if (!defined $data) {
 			$infoheader = "\n". _"No results found";
-			$artistinfo_ok = "0";
 			$tag_header = $tag_noresults;
 			$buffer->insert_with_tags($iter,$infoheader."\n",$tag_header);
 		} # fallback text if artist-info not found
-		else {	$artistinfo_ok = "1";
-			$buffer->insert_with_tags($iter,$warning,$tag_warning);
+		else {
+			for ($data) {
+				s/<br \/>|<\/p>/\n/gi;
+				s/\n\n*/\n/g; # never more than one empty line
+				s/<([^<]*)>//g; # strip tags
+			}
+			my $href = $buffer->create_tag(undef,justification=>'left',foreground=>"#4ba3d2",underline=>'single');
+			$href->{url}=$url.'/+wiki/edit';
+			my $aID = Songs::Get_gid($::SongID,'artist');
+			my $local_artist = Songs::Gid_to_Get("artist",$aID);
+			my $warning;
+			$warning= "Redirected to: ".$lfm_artist."\n" if $lfm_artist ne $local_artist;
+			$buffer->insert_with_tags($iter,$warning,$tag_warning) if $warning;
 			$buffer->insert_with_tags($iter,$infoheader."\n",$tag_header);
 			$buffer->insert($iter,$data);
 			$buffer->insert_with_tags($iter,"\n\n"._"Edit in the last.fm wiki",$href);
@@ -546,6 +568,7 @@ sub loaded
 			$self->{infoheader}=$infoheader;
 			$self->{biography}=$data;
 			$self->{lfm_url}=$url;
+			$self->Save_text if $::Options{OPT.'AutoSave'};
 		}
 	}
 
@@ -599,7 +622,6 @@ sub loaded
 
 		}
 	}
-	$self->Save_text if $::Options{OPT.'AutoSave'} && $artistinfo_ok && $artistinfo_ok==1;
 }
 
 sub load_file
