@@ -69,8 +69,8 @@ sub prefbox
 	my $check2=::NewPrefCheckButton(OPT.'UNIQUE',_"Find a unique filename if file already exists");
 	my $entry1=::NewPrefEntry(OPT.'COVERPATH');
 	my $entry2=::NewPrefEntry(OPT.'COVERFILE');
-	my ($radio1a,$radio1b)=::NewPrefRadio(OPT.'USEPATH',undef,_"use song folder",0,_"use :",1);
-	my ($radio2a,$radio2b)=::NewPrefRadio(OPT.'USEFILE',undef,_"use album name",0,_"use :",1);
+	my ($radio1a,$radio1b)=::NewPrefRadio(OPT.'USEPATH',[_"use song folder",0, _"use :",1]);
+	my ($radio2a,$radio2b)=::NewPrefRadio(OPT.'USEFILE',[_"use album name",0,  _"use :",1]);
 	my $frame1=Gtk2::Frame->new(_"default folder");
 	my $frame2=Gtk2::Frame->new(_"default filename");
 	my $vbox1=::Vpack( $radio1a,[$radio1b,$entry1] );
@@ -261,7 +261,7 @@ sub parse_freecovers #FIXME could use a XML module	#can provide backcover and mo
 sub parse_lastfm
 {	my $result=$_[0];
 	my @list;
-	while ($result=~m#<a href="/music/[^/]+/\+images/\d+"[^>]+?class="pic".+?<img [^>]+?src="([^"]+)"#gs)
+	while ($result=~m#<a\s+href="/music/[^/]+/\+images/\d+"[^>]+?class="pic".+?<img [^>]+?src="([^"]+)"#gs)
 	{	my $url=my $pre=$1;
 		$url=~s#/\w+/(\d+.jpg)$#/_/$1#; ### /126b/123456.jpg -> /_/123456.jpg
 		push @list, {url => $url, previewurl =>$pre,};
@@ -289,13 +289,13 @@ sub parse_sloth
 sub parse_googlei
 {	my $result=$_[0];
 	my @list;
-	while ($result=~m#<a [^>]*?href="([^"]+?imgrefurl[^"]+)"[^>]*>[^<]*<img [^>]*?src="([^"]+)">.*?</a>[^<]*<br>(.*?)<br>#g)
+	while ($result=~m#<a [^>]*?href="([^"]+?imgurl=[^"]+)"><img [^>]*?src="([^"]+)"[^>]*?>.*?</a>[^<]*<br/?>(.*?)<br/?/?>#g)
 	{	my ($url,$preview,$desc)=($1,$2,$3);
 		next unless $url=~m#imgurl=(.*?)&amp;#;
 		$url= ::decode_html($1);
 		$url=~s/%([0-9A-Fa-f]{2})/chr hex($1)/gie;
 		$desc=~s#</?b>##g;
-		$desc=Encode::decode('cp1252',$desc); #FIXME not sure of the encoding
+		$desc=Encode::decode('utf8',$desc);
 		$desc=::decode_html($desc);
 		push @list, {url => $url, previewurl =>$preview, desc => $desc };
 	}
