@@ -14,6 +14,7 @@ desc	Search and display lyrics
 package GMB::Plugin::LYRICS;
 use strict;
 use warnings;
+use utf8;
 require $::HTTP_module;
 our @ISA;
 BEGIN {push @ISA,'GMB::Context';}
@@ -62,11 +63,12 @@ my %Sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics
 	#lyricsplugin => [lyricsplugin => 'http://www.lyricsplugin.com/winamp03/plugin/?title=%t&artist=%a',undef,
 	#		sub { my $ok=$_[0]=~m#<div id="lyrics">.*\w\n.*\w.*</div>#s; $_[0]=~s/<div id="admin".*$//s if $ok; return $ok; }],
 	lyricssongs =>	['lyrics-songs',sub {  ::ReplaceFields($_[0], "http://letras.terra.com.br/winamp.php?musica=%t&artista=%a", sub {::url_escapeall(::superlc($_[0]));})  },undef,
-			sub {	my $l=html_extract($_[0],div=>'letra');
+			sub {	my $is_suggestion= $_[0]=~m#<h3>Provável música</h3>#i;
+				my $l=html_extract($_[0],div=>'letra');
 				$l=~s#<div id="cabecalho">.*?</div>##s if $l; #remove header with title and artist links
 				my $ref=\$_[0];
-				$$ref=$l ? $l : $notfound;
-				return ! !$l
+				$$ref= $l ? $l : $notfound;
+				return $l && !$is_suggestion;
 			}],
 	lyricwiki =>	[lyricwiki => 'http://lyrics.wikia.com/%a:%t',undef,
 			 sub {	return 0,'http://lyrics.wikia.com/'.$1 if $_[0]=~m#<span class="redirectText"><a href="/([^"]+)"#;
@@ -91,11 +93,12 @@ my $lyricswidget=
 {	class		=> __PACKAGE__,
 	tabicon		=> 'gmb-lyrics',		# no icon by that name by default
 	tabtitle	=> _"Lyrics",
-	saveoptions	=> 'HideToolbar font follow justification',
-	schange		=> \&SongChanged,
+	saveoptions	=> 'HideToolbar font follow justification edit',
+	schange		=> sub { $_[0]->SongChanged($_[1]); }, #$_[1] is new ID
 	group		=> 'Play',
 	autoadd_type	=> 'context page lyrics text',
 	justification	=> 'left',
+	edit		=> 0,
 };
 
 sub Start
@@ -109,10 +112,11 @@ sub new
 {	my ($class,$options)=@_;
 	my $self = bless Gtk2::VBox->new(0,0), $class;
 	$options->{follow}=1 if not exists $options->{follow};
-	$self->{$_}=$options->{$_} for qw/HideToolbar follow group font justification/;
+	$self->{$_}=$options->{$_} for qw/HideToolbar follow group font justification edit/;
 
 	my $textview=Gtk2::TextView->new;
 	$self->signal_connect(map => sub { $_[0]->SongChanged( ::GetSelID($_[0]) ); });
+	$self->signal_connect_after(key_press_event	=> \&key_pressed_cb);
 	$textview->signal_connect(button_release_event	=> \&button_release_cb);
 	$textview->signal_connect(motion_notify_event 	=> \&update_cursor_cb);
 	$textview->signal_connect(visibility_notify_event=>\&update_cursor_cb);
@@ -127,6 +131,7 @@ sub new
 	}
 	$self->{buffer}=$textview->get_buffer;
 	$self->{textview}=$textview;
+	$self->{DefaultFocus}=$textview;
 	my $sw=Gtk2::ScrolledWindow->new;
 	$sw->set_shadow_type( $options->{shadow} || 'etched-in');
 	$sw->set_policy('automatic','automatic');
@@ -152,6 +157,10 @@ sub new
 	# create follow toggle button, function from GMB::Context
 	my $follow=$self->new_follow_toolitem;
 
+	$self->{editb}=my $editmode= Gtk2::ToggleToolButton->new_from_stock('gtk-edit');
+	$editmode->signal_connect(toggled=> sub { SetEditable($_[0],$_[0]->get_active); });
+	$editmode->set_tooltip_text(_"Edit mode");
+
 	my $adj= $self->{fontsize_adj}= Gtk2::Adjustment->new(10,4,80,1,5,0);
 	my $zoom=Gtk2::ToolItem->new;
 	my $zoom_spin=Gtk2::SpinButton->new($adj,1,0);
@@ -159,7 +168,7 @@ sub new
 	$zoom->set_tooltip_text(_"Font size");
 	my $source=::NewPrefCombo( OPT.'LyricSite', { map {$_=>$Sites{$_}[0]} keys %Sites} ,cb => \&Refresh_cb, toolitem=> _"Lyrics source");
 	my $scroll=::NewPrefCheckButton( OPT.'AutoScroll', _"Auto-scroll", cb=>\&SetAutoScroll, tip=>_"Scroll with the song", toolitem=>1);
-	$toolbar->insert($_,-1) for $follow,$zoom,$scroll,$source;
+	$toolbar->insert($_,-1) for $editmode,$follow,$zoom,$scroll,$source;
 
 	$self->pack_start($toolbar,0,0,0);
 	$self->add($sw);
@@ -172,6 +181,7 @@ sub new
 	$zoom_spin->signal_connect(value_changed=> sub { my $self=::find_ancestor($_[0],__PACKAGE__); $self->SetFont($_[0]->get_value) });
 	$self->SetToolbarHide($self->{HideToolbar});
 	$self->SetAutoScroll;
+	$self->SetEditable($self->{edit});
 
 	return $self;
 }
@@ -217,6 +227,16 @@ sub SetToolbarHide
 	else				{$toolbar->set_no_show_all(0); $toolbar->show_all}
 }
 
+sub SetEditable
+{	my $self=::find_ancestor($_[0],__PACKAGE__);
+	my $on=$_[1];
+	$self->{edit}=$on;
+	my $view=$self->{textview};
+	$view->set_editable($on);
+	$view->set_cursor_visible($on);
+	$self->{editb}->set_active($on) if $self->{editb}->get_active xor $on;
+}
+
 sub SetAutoScroll
 {	my $self=::find_ancestor($_[0],__PACKAGE__);
 	if ($self->{AutoScroll}=$::Options{OPT.'AutoScroll'})
@@ -276,7 +296,7 @@ sub Back_cb
 }
 sub Refresh_cb
 {	my $self=::find_ancestor($_[0],__PACKAGE__);
-	$self->SongChanged($self->{ID},1);
+	$self->SongChanged($self->{ID},'force');
 }
 
 sub SongChanged
@@ -332,7 +352,7 @@ sub TimeChanged		#scroll the text
 	return unless $range >0;
 	return if $adj->get_value > $adj->upper - $adj->page_size;
 	my $delta=$::PlayTime - ($self->{time} || 0);
-	return if $delta <1;
+	return if abs($delta) <1;
 	my $inc=$delta / Songs::Get($::SongID,'length');
 	$self->{time}=$::PlayTime;
 	$adj->set_value($adj->get_value+$inc*$range);
@@ -534,7 +554,7 @@ sub loaded #_very_ crude html to gtktextview renderer
 	if ($oklyrics && $oklyrics>0)
 	{	$self->Save_text if $::Options{OPT.'AutoSave'};
 	}
-	else { $buffer->set_modified(0); }
+	#else { $buffer->set_modified(0); }
 }
 
 sub load_pixbuf
@@ -591,6 +611,23 @@ sub load_pixbuf
 #		else { $buffer->insert($iter, $_);}
 #	}
 #}
+my %zoomkeymap;
+BEGIN { %zoomkeymap=( plus=>1, minus=>-1, KP_Add=>1, KP_Subtract=>-1 ); }
+sub key_pressed_cb
+{	my ($self,$event)=@_;
+	my $key=Gtk2::Gdk->keyval_name( $event->keyval );
+	my $state=$event->get_state;
+	my $ctrl= $state * ['control-mask'] && !($state * [qw/mod1-mask mod4-mask super-mask/]); #ctrl and not alt/super
+	return 0 unless $ctrl; #only ctrl- shortcuts so far
+	if (lc$key eq 'e') { $self->SetEditable(!$self->{edit}); }
+	elsif (exists $zoomkeymap{$key})
+	{	my $size= $self->{fontsize_adj}->get_value;
+		$size+= $zoomkeymap{$key};
+		$self->{fontsize_adj}->set_value($size);
+	}
+	else {return 0}
+	return 1;
+}
 
 sub scroll_cb	#zoom with ctrl-wheel
 {	my ($textview,$event) = @_;
